@@ -62,76 +62,77 @@
 #' 
 #' ##test for Sex:Age interaction and Subject-Intercept
 #' mA<-lme(distance ~ Sex * I(age - 11), random = ~ 1| Subject,
-#'     data = Orthodont, method = "ML")
+#'   data = Orthodont, method = "ML")
 #' m0<-lm(distance ~ Sex + I(age - 11), data = Orthodont)
 #' summary(mA)
 #' summary(m0)
 #' exactLRT(m = mA, m0 = m0)
 #' 
 #' @export exactLRT
+#' @importFrom stats coefficients
 `exactLRT` <-
-    function(m, m0, seed = NA, nsim = 10000, 
-             log.grid.hi = 8, log.grid.lo = -10, gridlength = 200,
-             parallel = c("no", "multicore", "snow"), 
-             ncpus = 1L, cl = NULL) 
-    {
-        if (class(m0) != "lm") 
-            stop("m0 not an lm-object. \n")
-        if (class(m) == "spm") {
-            m <- m$fit
-            class(m) <- "lme"
-        }
-        if (class(m) %in% c("amer", "mer"))
-            stop("Package <amer> and versions of <lme4> below lme4_1.0 are no longer supported.")
-        if (!((c.m <- class(m)) %in% c("lme", "lmerMod", "merModLmerTest"))) 
-            stop("Invalid m specified. \n")
-        
-        
-        d <- switch(c.m, lme = extract.lmeDesign(m), 
-                    lmerMod=extract.lmerModDesign(m))
-        if(length(d$lambda) != 1 || d$k != 1) 
-            stop("multiple random effects in model - 
-                 exactLRT needs 'm' with only a single random effect.")
-        X <- d$X
-        Z <- d$Z
-        y <- d$y
-        Vr <- d$Vr
-        K <- NCOL(Z)
-        n <- NROW(X)
-        p <- NCOL(X)
-        q <- p - length(coefficients(m0)[!is.na(coefficients(m0))])
-        if (n != length(m0$fitted)) 
-            stop("different data under the null and alternative. \n")
-        if (q < 0) 
-            stop("m0 not nested in m. \n")
-        if (n - p - K < 1) 
-            stop("No. of effects greater than n. Reduce model complexity.\n")
-        if (q == 0) 
-            message("No restrictions on fixed effects. REML-based inference preferable.")
-        method <- switch(c.m, lme = m$method, 
-                         lmerMod=ifelse(isREML(m), "REML", "ML"))
-        if (method != "ML") {
-            message("Using likelihood evaluated at REML estimators.")
-            message("Please refit model with method=\"ML\" for exact results.")
-        }
-        #observed value of the LRT
-        lrt.obs <- max(0, 2 * logLik(m, REML = FALSE)[1] - 2 * logLik(m0, 
-                                                                      REML = FALSE)[1])
-        sample <- LRTSim(X, Z, q, sqrt.Sigma = chol(cov2cor(Vr)), 
-                         seed = seed, nsim = nsim, log.grid.hi = log.grid.hi, 
-                         log.grid.lo = log.grid.lo, gridlength = gridlength,
-                         parallel = match.arg(parallel), 
-                         ncpus = ncpus, cl = cl)
-        if (quantile(sample, 0.9) == 0) {
-            warning("Null distribution has", mean(sample == 
-                                                      0), "mass at zero.\n")
-        }
-        p <- mean(lrt.obs < sample)
-        RVAL <- list(statistic = c(LRT = lrt.obs), 
-                     p.value = p, 
-                     method = paste("simulated finite sample distribution of LRT. (p-value based on", 
-                                    nsim, "simulated values)"), sample=sample)
-        class(RVAL) <- "htest"
-        return(RVAL)
-    } 
+  function(m, m0, seed = NA, nsim = 10000, 
+    log.grid.hi = 8, log.grid.lo = -10, gridlength = 200,
+    parallel = c("no", "multicore", "snow"), 
+    ncpus = 1L, cl = NULL) 
+  {
+    if (class(m0) != "lm") 
+      stop("m0 not an lm-object. \n")
+    if (class(m) == "spm") {
+      m <- m$fit
+      class(m) <- "lme"
+    }
+    if (class(m) %in% c("amer", "mer"))
+      stop("Models fit with package <amer> or versions of <lme4> below 1.0 are no longer supported.")
+    if (!((c.m <- class(m)) %in% c("lme", "lmerMod", "merModLmerTest"))) 
+      stop("Invalid <m> specified. \n")
+    
+    
+    d <- switch(c.m, lme = extract.lmeDesign(m), 
+      lmerMod=extract.lmerModDesign(m))
+    if(length(d$lambda) != 1 || d$k != 1) 
+      stop("multiple random effects in model - 
+                 exactLRT needs <m> with only a single random effect.")
+    X <- d$X
+    Z <- d$Z
+    y <- d$y
+    Vr <- d$Vr
+    K <- NCOL(Z)
+    n <- NROW(X)
+    p <- NCOL(X)
+    q <- p - length(coefficients(m0)[!is.na(coefficients(m0))])
+    if (n != length(m0$fitted)) 
+      stop("different data under the null and alternative. \n")
+    if (q < 0) 
+      stop("m0 not nested in m. \n")
+    if (n - p - K < 1) 
+      stop("No. of effects greater than no. of observations. Reduce model complexity.\n")
+    if (q == 0) 
+      message("No restrictions on fixed effects. REML-based inference preferable.")
+    method <- switch(c.m, lme = m$method, 
+      lmerMod=ifelse(lme4::isREML(m), "REML", "ML"))
+    if (method != "ML") {
+      message("Using likelihood evaluated at REML estimators.")
+      message("Please refit model with method=\"ML\" for exact results.")
+    }
+    #observed value of the LRT
+    lrt.obs <- max(0, 2 * logLik(m, REML = FALSE)[1] - 2 * logLik(m0, 
+      REML = FALSE)[1])
+    sample <- LRTSim(X, Z, q, sqrt.Sigma = chol(cov2cor(Vr)), 
+      seed = seed, nsim = nsim, log.grid.hi = log.grid.hi, 
+      log.grid.lo = log.grid.lo, gridlength = gridlength,
+      parallel = match.arg(parallel), 
+      ncpus = ncpus, cl = cl)
+    if (quantile(sample, 0.9) == 0) {
+      warning("Null distribution has mass ", mean(sample == 
+          0), " at zero.\n")
+    }
+    p <- mean(lrt.obs < sample)
+    RVAL <- list(statistic = c(LRT = lrt.obs), 
+      p.value = p, 
+      method = paste("simulated finite sample distribution of LRT. (p-value based on", 
+        nsim, "simulated values)"), sample=sample)
+    class(RVAL) <- "htest"
+    return(RVAL)
+  } 
 
